@@ -3,6 +3,10 @@
 #define NPY_NO_DEPRECATED_API NPY_1_7_API_VERSION
 #include <numpy/ndarrayobject.h>
 
+#ifndef NPY_ARRAY_BEHAVED
+    #define NPY_ARRAY_BEHAVED NPY_BEHAVED
+#endif
+
 static
 int adapt_ndarray(PyObject *obj, void* arystruct) {
     PyArrayObject *ndary;
@@ -41,16 +45,38 @@ int adapt_ndarray(PyObject *obj, void* arystruct) {
     return 0;
 }
 
+static
+PyObject* ndarray_new(int nd,
+                      npy_intp *dims,   /* shape */
+                      npy_intp *strides,
+                      void* data,
+                      int type_num,
+                      int itemsize)
+{
+    PyObject *ndary;
+    int flags = NPY_ARRAY_BEHAVED;
+    ndary = PyArray_New((PyTypeObject*)&PyArray_Type, nd, dims, type_num,
+                       strides, data, 0, flags, NULL);
+    return ndary;
+}
+
 static
 PyObject* get_ndarray_adaptor(PyObject* self, PyObject *args)
 {
     return PyLong_FromVoidPtr(&adapt_ndarray);
 }
 
+static
+PyObject* get_ndarray_new(PyObject* self, PyObject *args)
+{
+    return PyLong_FromVoidPtr(&ndarray_new);
+}
+
 
 static PyMethodDef ext_methods[] = {
 #define declmethod(func) { #func , ( PyCFunction )func , METH_VARARGS , NULL }
     declmethod(get_ndarray_adaptor),
+    declmethod(get_ndarray_new),
     { NULL },
 #undef declmethod
 };
