@@ -16,23 +16,27 @@ registry = Registry()
 register = registry.register
 
 
+def _unary_int_input_wrapper_impl(wrapped_impl):
+    """
+    Return an implementation factory to convert the single integral input
+    argument to a float, then defer to the *wrapped_impl*.
+    """
+    def implementer(context, builder, sig, args):
+        [val] = args
+        input_type = sig.args[0]
+        if input_type.signed:
+            fpval = builder.sitofp(val, Type.double())
+        else:
+            fpval = builder.uitofp(val, Type.double())
+        sig = signature(types.float64, types.float64)
+        return wrapped_impl(context, builder, sig, [fpval])
+
+    return implementer
+
 def unary_math_int_impl(fn, f64impl):
-    @register
-    @implement(fn, types.int64)
-    def s64impl(context, builder, sig, args):
-        [val] = args
-        fpval = builder.sitofp(val, Type.double())
-        sig = signature(types.float64, types.float64)
-        return f64impl(context, builder, sig, [fpval])
-
-    @register
-    @implement(fn, types.uint64)
-    def u64impl(context, builder, sig, args):
-        [val] = args
-        fpval = builder.uitofp(val, Type.double())
-        sig = signature(types.float64, types.float64)
-        return f64impl(context, builder, sig, [fpval])
-
+    impl = _unary_int_input_wrapper_impl(f64impl)
+    for input_type in [types.intp, types.uintp, types.int64, types.uint64]:
+        register(implement(fn, input_type)(impl))
 
 def unary_math_intr(fn, intrcode):
     @register
@@ -56,26 +60,49 @@ def unary_math_intr(fn, intrcode):
     unary_math_int_impl(fn, f64impl)
 
 
-def unary_math_extern(fn, f32extern, f64extern):
-    @register
-    @implement(fn, types.float32)
-    def f32impl(context, builder, sig, args):
+def _float_input_unary_math_extern_impl(extern_func, input_type, restype=None):
+    """
+    Return an implementation factory to call unary *extern_func* with the
+    given argument *input_type*.
+    """
+    def implementer(context, builder, sig, args):
         [val] = args
         mod = cgutils.get_module(builder)
-        fnty = Type.function(Type.float(), [Type.float()])
-        fn = mod.get_or_insert_function(fnty, name=f32extern)
-        return builder.call(fn, (val,))
+        lty = context.get_value_type(input_type)
+        fnty = Type.function(lty, [lty])
+        fn = mod.get_or_insert_function(fnty, name=extern_func)
+        res = builder.call(fn, (val,))
+        if restype is None:
+            return res
+        else:
+            return context.cast(builder, res, input_type,
+                                restype)
 
-    @register
-    @implement(fn, types.float64)
-    def f64impl(context, builder, sig, args):
-        [val] = args
-        mod = cgutils.get_module(builder)
-        fnty = Type.function(Type.double(), [Type.double()])
-        fn = mod.get_or_insert_function(fnty, name=f64extern)
-        return builder.call(fn, (val,))
+    return implementer
 
-    unary_math_int_impl(fn, f64impl)
+def unary_math_extern(fn, f32extern, f64extern, int_restype=False):
+    """
+    Register implementations of Python function *fn* using the
+    external function named *f32extern* and *f64extern* (for float32
+    and float64 inputs, respectively).
+    If *int_restype* is true, then the function's return value should be
+    integral, otherwise floating-point.
+    """
+    f_restype = types.int64 if int_restype else None
+    f32impl = _float_input_unary_math_extern_impl(f32extern, types.float32, f_restype)
+    f64impl = _float_input_unary_math_extern_impl(f64extern, types.float64, f_restype)
+    register(implement(fn, types.float32)(f32impl))
+    register(implement(fn, types.float64)(f64impl))
+    
+    if int_restype:
+        # If asked for an integral return type, we choose the input type
+        # as the return type.
+        for input_type in [types.intp, types.uintp, types.int64, types.uint64]:
+            impl = _unary_int_input_wrapper_impl(
+                _float_input_unary_math_extern_impl(f64extern, types.float64, input_type))
+            register(implement(fn, input_type)(impl))
+    else:
+        unary_math_int_impl(fn, f64impl)
 
 
 unary_math_intr(math.fabs, lc.INTR_FABS)
@@ -101,10 +128,15 @@ unary_math_extern(math.atanh, "atanhf", "atanh")
 unary_math_extern(math.sinh, "sinhf", "sinh")
 unary_math_extern(math.cosh, "coshf", "cosh")
 unary_math_extern(math.tanh, "tanhf", "tanh")
-unary_math_extern(math.ceil, "ceilf", "ceil")
-unary_math_extern(math.floor, "floorf", "floor")
+# math.floor and math.ceil return float on 2.x, int on 3.x
+if utils.PYVERSION > (3, 0):
+    unary_math_extern(math.ceil, "ceilf", "ceil", True)
+    unary_math_extern(math.floor, "floorf", "floor", True)
+else:
+    unary_math_extern(math.ceil, "ceilf", "ceil")
+    unary_math_extern(math.floor, "floorf", "floor")
 unary_math_extern(math.sqrt, "sqrtf", "sqrt")
-unary_math_extern(math.trunc, "truncf", "trunc")
+unary_math_extern(math.trunc, "truncf", "trunc", True)
 
 
 @register
